/*

    Inline definitions for linear algebra operations

*/

#define X   0
#define Y   1
#define Z   2

typedef double Vector[3];

/*  In all cases the last argument is the result.  For example,
    VecSub(a, b, c) == c = a - b.  In the comments below, capital
    letters represent scalars, lower case 3-vectors.  */

/*  v = 0  */
#define VecZero(v)  	    (v)[X] = (v)[Y] = v[Z] = 0.0

/*  v = (x, y ,z)  */
#define VecSet(x, y, z, v)  (v)[X] = (x), (v)[Y] = (y), (v)[Z] = (z)

/*  a = S * a  */
#define VecScale(S,a)	    (a)[X] *= S ; (a)[Y] *= S ; (a)[Z] *= S

/*  a = -a  */
#define VecNegate(a)        (a)[X] = -(a)[X]; \
                            (a)[Y] = -(a)[Y]; \
			    (a)[Z] = -(a)[Z];
/*  a . b  */		    
#define VecDot(a,b)	    ((a)[X] * (b)[X] + (a)[Y] * (b)[Y] + (a)[Z] * (b)[Z])

/*  sqrt(a . a)  */
#define VecLen(a)	    (sqrt(VecDot(a, a)))

/*  a = a / sqrt(a . a)  */
#define VecNorm(a)  	    { double sf  =  1.0 / VecLen(a); VecScale(sf, a); }

/*  b = a  */
#define VecCopy(a,b)	    (b)[X] = (a)[X];(b)[Y] = (a)[Y];(b)[Z] = (a)[Z];

/*  c = a + b  */
#define VecAdd(a,b,c)       (c)[X] = (a)[X] + (b)[X]; \
                            (c)[Y] = (a)[Y] + (b)[Y]; \
			    (c)[Z] = (a)[Z] + (b)[Z]

/*  c = a - b  */			    
#define VecSub(a,b,c)       (c)[X] = (a)[X] - (b)[X]; \
                            (c)[Y] = (a)[Y] - (b)[Y]; \
    	    	    	    (c)[Z] = (a)[Z] - (b)[Z]

/*  c = (A * a) + (B * b)  */			    
#define VecComb(A,a,B,b,c)  (c)[X] = (A) * (a)[X] + (B) * (b)[X]; \
                    	    (c)[Y] = (A) * (a)[Y] + (B) * (b)[Y]; \
			    (c)[Z] = (A) * (a)[Z] + (B) * (b)[Z]

/*  c = (A * a) + b  */	    
#define VecAddS(A,a,b,c)    (c)[X] = (A) * (a)[X] + (b)[X]; \
    	    	    	    (c)[Y] = (A) * (a)[Y] + (b)[Y]; \
    	    	    	    (c)[Z] = (A) * (a)[Z] + (b)[Z]

/*  c = a  b  */    
#define VecCross(a,b,c)     (c)[X] = (a)[Y] * (b)[Z] - (a)[Z] * (b)[Y]; \
    	    	    	    (c)[Y] = (a)[Z] * (b)[X] - (a)[X] * (b)[Z]; \
    	    	    	    (c)[Z] = (a)[X] * (b)[Y] - (a)[Y] * (b)[X]

/*  Print "msg" followed by components of v  */			    
#define VecPrint(msg,v)     printf("%s %g %g %g\n", msg,\
			    	(v)[X], (v)[Y], (v)[Z])
