
    #	Generate simultaneous transit HTML catalogue
    
    #	Input:	simultaneous.csv    (Created by simultaneous_csv.pl)
    #	Output: simultaneous.html
    
    #	by John Walker
    #	http://www.fourmilab.ch/
    
    #	For details, please see: http://www.fourmilab.ch/documents/canon_transits/
    

    use strict;
    use warnings;
    require 'julian.pl';
    
    my $il;
    
    my @planets = (
    	    	    	'Mercury',
			'Venus',
			'Earth',
			'Moon',
			'Mars',
			'Jupiter',
			'Saturn',
			'Uranus',
			'Neptune',
			'Pluto'
    	    	  );
		  
    my %porder = (
   	    	    	'Mercury', 1,
			'Venus', 2,
			'Earth', 3,
			'Moon', 4,
			'Mars', 5,
			'Jupiter', 6,
			'Saturn', 7,
			'Uranus', 8,
			'Neptune', 9,
			'Pluto', 10
     	    	 );
		  
    
    my $limitLines = 0;	    	    # Only read this many lines from transit canon (0 = all)
#$limitLines = 20000;
    my $lineno = 0;
    
    my $tsimultaneous = `wc -l simultaneous.csv`;   # Now is this a brutal hack or what?
    $tsimultaneous =~ s/\D//g;
    
    my (%multfrom, %triplefrom, %from_of);  # Event occurrence counting hashes
    
    open(IX, ">simultaneous.html") || die "Cannot open simultaneous.html";
    
    makeHTMLheader();
    
    open(FI, "<simultaneous.csv");
    while ($il = <FI>) {
    	$lineno++;
	if ($lineno eq $limitLines) {
	    last;
	}
	my $l = $il;
    	$l =~ s/\s+//;
	$l =~ s/^([^\,]+),([^\,]+),([^\,]+),([^\,]+),/,/ ||
	    die("Parse error 1 on $il");
#print("$1 $2 $3 $4\n");
    	my ($from, $mult_first, $mult_last, $nbodies) = ($1, $2, $3, $4);
	
	my (@of, @first, @max, @last);
	
	my $tsig = "$from";
	
	for (my $i = 0; $i < $nbodies; $i++) {
	    $l =~ s/^,([^\,]+),([^\,]+),([^\,]+),([^\,]+)// ||
	    	die("Parse error 2 on $il");
#print("  $i:  $1 $2 $3 $4\n");
    	    $tsig .= ":$1";
	    push(@of, $1);
	    push(@first, $2);
	    push(@max, $3);
	    push(@last, $4);
	}
	
	$multfrom{$from}++;
	if ($nbodies > 2) {
	    $triplefrom{$from}++;
	}

    	#   Sort planets in transit into canonical order
#print("Was: ($tsig)  ");
	my @t = split(/:/, $tsig);
	my $t1 = shift(@t);
	@t = sort { $porder{$a} <=> $porder{$b} } @t;
	$tsig = "$t1:" . join(':', @t);
#print("Is: ($tsig)\n");
	$from_of{$tsig}++;

    	printf(IX "%-8s     %s  %s\n", $from, JDwC($mult_first), JDwC($mult_last));
	for (my $i = 0; $i <= $#of; $i++) {
	    printf(IX "    %-8s %s  %s\n", $of[$i], JDwC($first[$i]), JDwC($last[$i]));
	}
    }
    close(FI);
    
    die("Lines ($lineno) read from simultaneous.csv disagress with line count ($tsimultaneous)")
    	if ($lineno != $tsimultaneous);
	
    print IX <<"EOD";
</pre>

<p>

EOD
	
    generateMultipleTables();
    
    print IX <<"EOD";
<p>
&nbsp;
</div>
</body>
</html>
EOD
    
    close(IX);

    my ($i, $j);
    
    sub JDwC {
    	my ($j) = @_;
	
    	return sprintf("%12.2f %21s", $j, JDtoCalendar($j));
    }
    
    #	Generate tables for multiple transits
    
    sub generateMultipleTables {
    	my $k;
	
	#   Multiple transits by frequency of from planet
	
    	print IX << "EOD";
<table border="border" bgcolor="#E0E0E0" align="right" hspace="12" cellpadding="3">
<tr><th>Multiple Transits<th>Visible From
EOD

	foreach $k (sort { $multfrom{$a} <=> $multfrom{$b} } keys(%multfrom)) {
	    print(IX "<tr><td align=\"center\">$multfrom{$k}<td align=\"center\">$k\n");
	}
	print(IX "</table>\n");
	
	#   Triple transits by frequency of from planet
	
    	print IX << "EOD";
<table border="border" bgcolor="#E0E0E0" align="left" hspace="12" cellpadding="3">
<tr><th>Triple Transits<th>Visible From
EOD

	foreach $k (sort { $triplefrom{$a} <=> $triplefrom{$b} } keys(%triplefrom)) {
	    print(IX "<tr><td align=\"center\">$triplefrom{$k}<td align=\"center\">$k\n");
	}
	print(IX "</table>\n");
	
	print(IX "<br clear=\"all\">\n<p>\n");
	
	#   Frequency by from planet and planets in transit
	
    	print IX << "EOD";
<table border="border" bgcolor="#E0E0E0" align="center" cellpadding="3">
<tr><th>Occurrences<th>From<th>Transit Of
    <td class="s">&nbsp;<th>Occurrences<th>From<th>Transit Of
EOD

    	my @ttab;
	my $vsum = 0;
	foreach $k (sort { $a =~ m/^(\w+):(\w+):/; 
	    	    	   my ($pa, $ta) = ($1, $2);
			   $b =~ m/^(\w+):(\w+):/; 
	    	    	   my ($pb, $tb) = ($1, $2);
			   ($from_of{$a} == $from_of{$b}) ?
			    	(($porder{$pa} == $porder{$pb}) ?
				    ($porder{$ta} <=> $porder{$tb}) :
				    ($porder{$pa} <=> $porder{$pb})) :
				($from_of{$a} <=> $from_of{$b}) }
		    keys(%from_of)) {
	    $k =~ m/^(\w+):(.*)$/;
	    my ($from, $of) = ($1, $2);
	    $of =~ s/:/ /g;
	    push(@ttab,"<td align=\"center\">$from_of{$k}<td align=\"center\">$from<td align=\"center\">$of");
	    $vsum += $from_of{$k};
	}
    	die("Lines ($lineno) read from simultaneous.csv disagress with transit count ($vsum)")
    	    if ($lineno != $vsum);

	my $httabl = int(($#ttab) / 2);
#print("TTabl = $#ttab  Httabl = $httabl  Last = $ttab[$#ttab]\n");	
	for (my $i = 0; $i <= $httabl; $i++) {
	    my $s = $ttab[$i];
	    print(IX "<tr>$s\n");
	    	print(IX "    <td class=\"s\">&nbsp;");
	    if ((($i * 2) + 1) <= $#ttab) {
	    	$s = $ttab[$httabl + $i + 1];
	    	print(IX "$s\n");
	    } else {
	    	print(IX "<td>&nbsp;<td>&nbsp;<td>&nbsp;\n");
	    }
	}
	
	print(IX "</table>\n");

    }

    #	Create HTML file header
    
    sub makeHTMLheader {
    	print IX << "EOD";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<html version="-//W3C//DTD HTML 3.2 Final//EN">
<head>
<title>Simultaneous Transits: 125,000 Years</title>
<link rel="stylesheet" type="text/css"
      href="http://www.fourmilab.ch/documents/canon_transits/canon.css">
<meta name="robots" content="noindex,nofollow">
</head>
<body bgcolor="#FFFFFF">
<div class="bodycopy">
<center>
<h1>Simultaneous Transits: 125,000 Years</h1>
</center>

The following table lists all $tsimultaneous simultaneous transits in the
<cite><a href="http://www.fourmilab.ch/documents/canon_transits/">Quarter
Million Year Canon of Solar System Transits</a></cite>.  A
simultaneous transit is defined as one in which two or more planets
are in transit across the Sun simultaneously as seen from an outer
planet, but excludes those where the Earth and Moon are the only two
bodies in transit.

<p>

Fields in this table are as follows:

<p>
<table align="center" border="2" bgcolor="#E0E0E0" cellpadding="3">
<tr><th valign="top" align="left">From/Of
    <td valign="top" >The name of the planet from which the transit is
    	visible is the first line of each entry.  Planets
	which transit the Sun appear on subsequent lines,
	indented.
<tr><th  valign="top" align="left">First JD
    <td valign="top" >Julian date when transit began.  On the first line, this is
    	the date of the start of the multiple transit.  On subsequent lines,
	it is the date when that planet began its transit.
<tr><th valign="top" align="left">First Date Time
    <td valign="top" >Date and time corresponding to <b>First JD</b>.
<tr><th align="left">C
    <td valign="top" >Calendar: <b>J</b> = Julian, <b>G</b> = Gregorian
    
<tr><th valign="top" align="left">Last JD
    <td valign="top" >Julian date when transit ended.  On the first line, this is
    	the date of the end of the multiple transit.  On subsequent lines,
	it is the date when that planet ended its transit.
<tr><th valign="top" align="left">Last Date Time
    <td valign="top" >Date and time corresponding to <b>Last JD</b>.
<tr><th valign="top" align="left">C
    <td valign="top">Calendar: <b>J</b> = Julian, <b>G</b> = Gregorian
</table>
<p>

Note that dates and times are the integration step time when the transit
was first and last observed, not the precise moment when the transit
began and ended.  Integration steps are 1/100 day, 14.4 minutes.

<pre>
From/Of        First JD     First Date  Time  C    Last JD       Last Date  Time  C
=======      ============  ================== =  ============  ================== =
EOD
    }
